/*
 * pngfp8.c - wrapper functions for floating-point
 ( m6888x version)
 *
 * Copyright (C) 2018 Thorsten Otto
 *
 * This code is released under the libpng license.
 * For conditions of distribution and use, see the disclaimer
 * and license in png.h
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "png.h"
#include "pcmath.h"

#if !defined(__HAVE_68881__)
# error "this must be compiled for 68881"
#endif

#pragma warn -stv

void PNGAPI png_pc8_set_rgb_to_gray(png_structrp png_ptr, png_int_t error_action, double red, double green)
{
	(png_set_rgb_to_gray)(png_ptr, error_action, pc_d96tod64(red), pc_d96tod64(green));
}


void PNGAPI png_pc8_set_background(png_structrp png_ptr, png_const_color_16p background_color, png_int_t background_gamma_code,
    png_int_t need_expand, double background_gamma)
{
	(png_set_background)(png_ptr, background_color, background_gamma_code, need_expand, pc_d96tod64(background_gamma));
}


void PNGAPI png_pc8_set_gamma(png_structrp png_ptr, double screen_gamma, double override_file_gamma)
{
	(png_set_gamma)(png_ptr, pc_d96tod64(screen_gamma), pc_d96tod64(override_file_gamma));
}


void PNGAPI png_pc8_set_filter_heuristics(png_structrp png_ptr, png_int_t heuristic_method, png_int_t num_weights, png_const_doublep filter_weights, png_const_doublep filter_costs)
{
	/* deprecated */
	(void)png_ptr;
	(void)heuristic_method;
	(void)num_weights;
	(void)filter_weights;
	(void)filter_costs;
}


float PNGAPI png_pc8_get_pixel_aspect_ratio(png_const_structrp png_ptr, png_const_inforp info_ptr)
{
	return pc_float_from_gcc8((png_get_pixel_aspect_ratio)(png_ptr, info_ptr));
}


png_uint_32 PNGAPI png_pc8_get_cHRM(png_const_structrp png_ptr, png_const_inforp info_ptr, double *white_x, double *white_y, double *red_x,
    double *red_y, double *green_x, double *green_y, double *blue_x, double *blue_y)
{
	__gcc_double_t gwhite_x;
	__gcc_double_t gwhite_y;
	__gcc_double_t gred_x;
	__gcc_double_t gred_y;
	__gcc_double_t ggreen_x;
	__gcc_double_t ggreen_y;
	__gcc_double_t gblue_x;
	__gcc_double_t gblue_y;
	png_uint_32 ret;
	
	ret = (png_get_cHRM)(png_ptr, info_ptr, &gwhite_x, &gwhite_y, &gred_x, &gred_y, &ggreen_x, &ggreen_y, &gblue_x, &gblue_y);
	if (white_x)
		*white_x = pc_d64tod96(gwhite_x);
	if (white_y)
		*white_y = pc_d64tod96(gwhite_y);
	if (red_x)
		*red_x = pc_d64tod96(gred_x);
	if (red_y)
		*red_y = pc_d64tod96(gred_y);
	if (green_x)
		*green_x = pc_d64tod96(ggreen_x);
	if (green_y)
		*green_y = pc_d64tod96(ggreen_y);
	if (blue_x)
		*blue_x = pc_d64tod96(gblue_x);
	if (blue_y)
		*blue_y = pc_d64tod96(gblue_y);
	return ret;
}


void PNGAPI png_pc8_set_cHRM(png_const_structrp png_ptr, png_inforp info_ptr,
    double white_x, double white_y, double red_x, double red_y, double green_x,
    double green_y, double blue_x, double blue_y)
{
	(png_set_cHRM)(png_ptr, info_ptr, pc_d96tod64(white_x), pc_d96tod64(white_y), pc_d96tod64(red_x), pc_d96tod64(red_y),
		pc_d96tod64(green_x), pc_d96tod64(green_y), pc_d96tod64(blue_x), pc_d96tod64(blue_y));
}


png_uint_32 PNGAPI png_pc8_get_gAMA(png_const_structrp png_ptr, png_const_inforp info_ptr, double *file_gamma)
{
	__gcc_double_t gamma;
	png_uint_32 ret;
	
	ret = (png_get_gAMA)(png_ptr, info_ptr, &gamma);
	if (file_gamma)
		*file_gamma = pc_d64tod96(gamma);
	return ret;
}


void PNGAPI png_pc8_set_gAMA(png_const_structrp png_ptr, png_inforp info_ptr, double file_gamma)
{
	(png_set_gAMA)(png_ptr, info_ptr, pc_d96tod64(file_gamma));
}


png_uint_32 PNGAPI png_pc8_get_sCAL(png_const_structrp png_ptr, png_const_inforp info_ptr, png_int_t *unit, double *width, double *height)
{
	png_uint_32 ret;
	__gcc_double_t w, h;
	
	ret = (png_get_sCAL)(png_ptr, info_ptr, unit, &w, &h);
	if (width)
		*width = pc_d64tod96(w);
	if (height)
		*height = pc_d64tod96(h);
	return ret;
}


void PNGAPI png_pc8_set_sCAL(png_const_structrp png_ptr, png_inforp info_ptr, png_int_t unit, double width, double height)
{
	(png_set_sCAL)(png_ptr, info_ptr, unit, pc_d96tod64(width), pc_d96tod64(height));
}


float PNGAPI png_pc8_get_x_offset_inches(png_const_structrp png_ptr, png_const_inforp info_ptr)
{
	return pc_float_from_gcc8((png_get_x_offset_inches)(png_ptr, info_ptr));
}


float PNGAPI png_pc8_get_y_offset_inches(png_const_structrp png_ptr, png_const_inforp info_ptr)
{
	return pc_float_from_gcc8((png_get_y_offset_inches)(png_ptr, info_ptr));
}


void PNGAPI png_pc8_set_alpha_mode(png_structrp png_ptr, png_int_t mode, double output_gamma)
{
	(png_set_alpha_mode)(png_ptr, mode, pc_d96tod64(output_gamma));
}


png_uint_32 PNGAPI png_pc8_get_cHRM_XYZ(png_const_structrp png_ptr,
    png_const_inforp info_ptr, double *red_X, double *red_Y, double *red_Z,
    double *green_X, double *green_Y, double *green_Z, double *blue_X,
    double *blue_Y, double *blue_Z)
{
	__gcc_double_t gred_x, gred_y, gred_z;
	__gcc_double_t ggreen_x, ggreen_y, ggreen_z;
	__gcc_double_t gblue_x, gblue_y, gblue_z;
	png_uint_32 ret;
	
	ret = (png_get_cHRM_XYZ)(png_ptr, info_ptr, &gred_x, &gred_y, &gred_z, &ggreen_x, &ggreen_y, &ggreen_z, &gblue_x, &gblue_y, &gblue_z);
	if (red_X)
		*red_X = pc_d64tod96(gred_x);
	if (red_Y)
		*red_Y = pc_d64tod96(gred_y);
	if (red_Z)
		*red_Z = pc_d64tod96(gred_z);
	if (green_X)
		*green_X = pc_d64tod96(ggreen_x);
	if (green_Y)
		*green_Y = pc_d64tod96(ggreen_y);
	if (green_Z)
		*green_Z = pc_d64tod96(ggreen_z);
	if (blue_X)
		*blue_X = pc_d64tod96(gblue_x);
	if (blue_Y)
		*blue_Y = pc_d64tod96(gblue_y);
	if (blue_Z)
		*blue_Z = pc_d64tod96(gblue_z);
	return ret;
}


void PNGAPI png_pc8_set_cHRM_XYZ(png_const_structrp png_ptr,
    png_inforp info_ptr, double red_X, double red_Y, double red_Z,
    double green_X, double green_Y, double green_Z, double blue_X,
    double blue_Y, double blue_Z)
{
	(png_set_cHRM_XYZ)(png_ptr, info_ptr,
		pc_d96tod64(red_X), pc_d96tod64(red_Y), pc_d96tod64(red_Z),
		pc_d96tod64(green_X), pc_d96tod64(green_Y), pc_d96tod64(green_Z),
		pc_d96tod64(blue_X), pc_d96tod64(blue_Y), pc_d96tod64(blue_Z));
}
